package org.netkernel.lang.dpml.ast.impl;

import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.endpoint.CompilerScope;
import org.netkernel.lang.dpml.endpoint.DPMLRuntime;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.request.IRequestScopeLevel;

import java.util.*;


public class Closure extends VariableImpl implements IScope, IValueVariable
{
	private Map mAssignments;
	private List mImports = new ArrayList();
	private CompilerScope mCompilerScope;
	
	public static final int PREC_LOW=-1;
	public static final int PREC_SAME=0;
	public static final int PREC_HIGH=1;
	
	public static final String RESPONSE="response";
	
	public Closure(String aName, IScope aScope, ILocation aLocation, CompilerScope aCompilerScope)
	{	super(aName,aScope,aLocation);
		mAssignments = new HashMap();
		mCompilerScope=aCompilerScope;
	}

	public IScope getParent()
	{	return getScope();
	}
	
	public void addAssignment(IVariable aVariable, boolean aReplace) throws NKFException
	{	IVariable existingAssignment  =(IVariable)mAssignments.put(aVariable.getName(),aVariable);
		if (existingAssignment!=null && !aReplace)
		{	throw new DuplicateAssignmentException();
		}
	}

	public void setResponse(IVariable aResponse)
	{	mAssignments.put(RESPONSE,aResponse);
	}
	
	public IVariable resolveVariable(String aName, IRequestScopeLevel aDynamicScope)
	{	return (IVariable)mAssignments.get(aName);
	}
	
	public Collection<IVariable> getVariables()
	{	return mAssignments.values();
	}

	public IValue getValue(INKFRequestContext aContext, IRequestScopeLevel aSubRequestContext, Class aRepresentationClass) throws NKFException
	{	
		IValue result;
		/*
		if (aContext.shouldLog(INKFRequestContext.LEVEL_INFO))
		{	System.out.println("executing closure "+getName());
		}
		*/
		
		IVariable resp = resolveVariable(RESPONSE, aSubRequestContext);
		
		if (resp==null)
		{	if (mAssignments.size()==1)
			{	resp= (IVariable)mAssignments.values().iterator().next();
			}
			else
			{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_PXML_NO_RESPONSE", DPMLRuntime.formatLocationMessage(this,aContext),null);
			}
		}		
		resp = DPMLRuntime.dereference(resp,aContext,aSubRequestContext);
		
		if (resp instanceof IIdentifierVariable)
		{	
			IIdentifierVariable idVar = (IIdentifierVariable)resp;
			String id=idVar.getIdentifier(aContext,aSubRequestContext);
			INKFRequest req=aContext.createRequest(id);
			req.setVerb(aContext.getThisRequest().getVerb());
			req.setRequestScope(aSubRequestContext);
			req.setRepresentationClass(aRepresentationClass);
			INKFResponseReadOnly r=aContext.issueRequestForResponse(req);
			result = new ValueImpl(r.getRepresentation(),((NKFResponseReadOnlyImpl)r).getMeta());
		}
		else
		{	result= ((IValueVariable)resp).getValue(aContext,aSubRequestContext,aRepresentationClass);
		}
		return result;
	}	
	
	public void addImport(IScope aImport, int aPrecedence)
	{	ImportDef importDef = new ImportDef(aImport,aPrecedence);
		mImports.add(importDef);
	}
	
	public void close(INKFRequestContext aContext) throws NKFException
	{
		if (mImports!=null)
		{
			for (Iterator i=mImports.iterator(); i.hasNext(); )
			{	ImportDef importDef = (ImportDef)i.next();
				for (Iterator j=importDef.mRootScope.getVariables().iterator(); j.hasNext(); )
				{	IVariable var = (IVariable)j.next();
					boolean existingVar = mAssignments.get(var.getName())!=null;
					if (!existingVar)
					{	addAssignment(var,false);
					}
					else
					{	switch (importDef.mPrecedence)
						{	case PREC_SAME:
								throw aContext.createFormattedException("EX_PXML_PARSE","MSG_PXML_IMPORT_CONFLICT",null,null,
										var.getName(), var.getLocation().getSourceIdentity() );
							case PREC_LOW:
								break; // ignore the imported defn
							case PREC_HIGH:
								addAssignment(var,true); // replace existing
								break;
						}
					}
				}
			}
			mImports=null;
		}
		
		
	}
	
	public CompilerScope getCompilerScope()
	{	return mCompilerScope;
	}
	
	public String toString()
	{	return "Closure["+getName()+"]";
	}
	
	private static class ImportDef
	{
		public final IScope mRootScope;
		public final int mPrecedence;
		
		public ImportDef(IScope aRootScope, int aPrecedence)
		{	mRootScope=aRootScope;
			mPrecedence=aPrecedence;
		}
	}

}
