package org.netkernel.lang.dpml.ast.impl;

import org.netkernel.lang.dpml.ast.ILocation;
import org.netkernel.lang.dpml.ast.IScope;
import org.netkernel.lang.dpml.ast.IVariable;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;

public class VariableImpl implements IVariable
{
	private final String mName;
	private final IScope mScope;
	private String mId;
	private final ILocation mLocation;
	private static int sId=0;
	
	public VariableImpl(String aName, IScope aScope, ILocation aLocation)
	{	mName=aName;
		mScope=aScope;
		mLocation=aLocation;
		String source = aLocation.getSourceIdentity();
		int j=source.lastIndexOf('/');
		if (j>0)
		{	source=source.substring(j+1);
		}
		mId=source+"(";
		if (mName!=null)
		{	mId+=mName;
		}
		else
		{	mId+=generateUniqueId();
		}
		mId+=")";
		//System.out.println("variableImpl "+mId+" "+mName+" "+toString());
	}
	
	public String getName()
	{	return mName;
	}
	
	public String getId()
	{	return mId;
	}
	
	private String generateUniqueId()
	{	int j;
		synchronized(VariableImpl.class)
		{	j=sId++;
		}
		return Integer.toHexString(j);
	}

	public IScope getScope()
	{	return mScope;
	}

	public ILocation getLocation()
	{	return mLocation;
	}
	
	public void close(INKFRequestContext aContext) throws NKFException
	{
	}

}
